// Setting up some stuff to play around with.

world
	view = 12
	fps = 50


atom/icon = 'icons/demo.dmi'

turf
	icon_state = "turf"

	wall
		icon_state = "wall"
		density = 1
		opacity = 1

	dirt
		icon_state = "dirt"

var/obj/light/npc_light // Since they're not individually manipulated, NPCs can all share a single global light object.

// You could make all things share a single light object, but you'd lose the ability to change the size, color, etc... without it effecting all lights.

obj/cpu_display
	plane = HUD_PLANE
	screen_loc = "1,1"
	var/shown = 0
	maptext_width = 128

	proc/Update()
		while(shown)
			maptext = "<font style=\"color:white; -dm-text-outline: 1px navy;\">CPU: [world.cpu]% - TU: [world.tick_usage]</font>"
			sleep(10)

mob
	icon_state = "player"
	appearance_flags = LONG_GLIDE
	glide_size = 8
	New()
		..()
		MoveLoop()
	var
		tmp
			zoomed = 0
			screen_angle = 0
			obj/cpu_display/cpu_display
			list/keys_pressed

	proc
		MoveLoop()
			set waitfor = 0
			set background = 1
			while(src)
				if(keys_pressed && keys_pressed.len)
					var/move_dir = 0
					if(keys_pressed["W"] || keys_pressed["NORTH"]) move_dir |= NORTH
					if(keys_pressed["S"] || keys_pressed["SOUTH"]) move_dir |= SOUTH
					if(keys_pressed["D"] || keys_pressed["EAST"]) move_dir |= EAST
					if(keys_pressed["A"] || keys_pressed["WEST"]) move_dir |= WEST
					step(src,move_dir)
				sleep((32/glide_size*world.tick_lag))


	verb

		KeyDown(button as text)
			set hidden = 1
			set instant = 1
			if(!keys_pressed) keys_pressed = list()
			button = uppertext(button)
			if(keys_pressed[button]) return
			keys_pressed[button] = 1
		KeyUp(button as text)
			set hidden = 1
			set instant = 1
			if(!keys_pressed) keys_pressed = list()
			button = uppertext(button)
			if(!keys_pressed[button]) return
			keys_pressed -= button
		// Keep in mind, we're resetting transform to null in these.
		// If you wanted changes to persist across transforms you'd keep the value of the variable to set it back to that instead of null.
		CPU()
			if(!cpu_display) cpu_display = new()
			cpu_display.shown = !cpu_display.shown
			if(!cpu_display.shown) client.screen -= cpu_display
			else
				client.screen += cpu_display
				cpu_display.Update()
			// We'll use a CPU display thingy to show how to use a plane above the darkness.

		Zoom()
			// This will zoom/unzoom everything on the darkness and master planes.
			// But it won't zoom things on the HUD plane.
			zoomed = !zoomed
			if(zoomed)
				// You can animate() planes just like anything else.
				animate(master_plane,transform = matrix()*2,time=5)
				animate(darkness_plane,transform = matrix()*2,time=5)
			else
				animate(master_plane,transform = null,time=5)
				animate(darkness_plane,transform = null,time=5)


		ToggleLight()
			// You could also animate() shrinking the light down to nothing if you wanted to be fancy.
			if((light in vis_contents)) vis_contents -= light
			else vis_contents += light

		LightSize(n as num)
			// Does the same work as light/New() for scaling the light up/down.
			light.transform = matrix(1,0,light.scale_number,0,1,light.scale_number)
			if(n != 1)
				light.transform = matrix(n, MATRIX_SCALE) * light.transform

		LightColor(c as color)
			if(isnull(c)) return
			light.color =  c

		LightAlpha(n as num)
			light.alpha = n

		LightPX(n as num)
			light.pixel_x = n

		LightPY(n as num)
			light.pixel_y = n

		NPC_LightSize(n as num)
			// Showing off updating all NPC lights by changing the global light they share.
			if(!npc_light) return
			npc_light.transform = matrix(1,0,npc_light.scale_number,0,1,npc_light.scale_number)
			if(n != 1)
				npc_light.transform = matrix(n, MATRIX_SCALE) * npc_light.transform

		NPC_LightColor(c as color)
			if(isnull(c)) return
			if(!npc_light) return
			npc_light.color = c

		TurnScreen()
			// Even Turn() transforms work on planes.
			var/matrix/turn_matrix = matrix()
			screen_angle += 45
			if(screen_angle > 360) screen_angle = 0
			turn_matrix.Turn(screen_angle)
			animate(master_plane,transform=turn_matrix,time=5)
			animate(darkness_plane,transform=turn_matrix,time=5)

		DarknessLevel(n as num)
			darkness_plane.color = list(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,0, n/10,n/10,n/10,1)

		// You can also color planes to accomplish similar effects as manipulation of client.color.
		// This could potentially be used to add objects to the map that change how the client sees the game when they're visible to it.
		// (think a mob that distorts the client screen anytime it's within view)

		ViewSize(n as num)
			client.view = n

		CreateDummy()
			new/mob/Dummy(loc)

		CreateDummy_Moving()
			new/mob/Dummy/Moving(loc)


	// Something to populate the map
	Dummy
		New()
			..()
			if(!npc_light) npc_light = new()
			vis_contents += npc_light

		Moving
			New()
				..()
				walk_rand(src,(32/glide_size*world.tick_lag))